@@ -34,10 +34,11 @@ from pyTEMlib.config_dir import config_path
 from pyTEMlib.sidpy_tools import *
 
 from pyTEMlib.sidpy_tools import *
+
 Qt_available = True
 try:
     from PyQt5 import QtCore, QtWidgets, QtGui
-except ModuleNotFoundError :
+except ModuleNotFoundError:
     print('Qt dialogs are not available')
     Qt_available = False
 
@@ -150,6 +151,7 @@ class ChooseDataset(object):
     """Widget to select dataset object """
 
     def __init__(self, input_object, show_dialog=True):
+        self.datasets = None
         if isinstance(input_object, sidpy.Dataset):
             if isinstance(input_object.h5_dataset, h5py.Dataset):
                 self.current_channel = input_object.h5_dataset.parent
@@ -157,17 +159,21 @@ class ChooseDataset(object):
             self.current_channel = input_object
         elif isinstance(input_object, h5py.Dataset):
             self.current_channel = input_object.parent
+        elif isinstance(input_object, dict):
+            self.datasets = input_object
         else:
             raise ValueError('Need hdf5 group or sidpy Dataset to determine image choices')
         self.dataset_names = []
         self.dataset_list = []
         self.dataset_type = None
         self.dataset = None
-        self.reader = SciFiReaders.NSIDReader(self.current_channel.file.filename)
-
+        if not isinstance(self.datasets, dict):
+            self.reader = SciFiReaders.NSIDReader(self.current_channel.file.filename)
+        else:
+            self.reader = None
         self.get_dataset_list()
-        self.select_image = widgets.Dropdown(options=self.dataset_names,
-                                             value=self.dataset_names[0],
+        self.select_image = widgets.Dropdown(options=self.dataset_list,
+                                             value=self.dataset_list[0],
                                              description='select dataset:',
                                              disabled=False,
                                              button_style='')
@@ -180,24 +186,28 @@ class ChooseDataset(object):
 
     def get_dataset_list(self):
         """ Get by Log number sorted list of datasets"""
-        datasets = self.reader.read()
+        if not isinstance(self.datasets, dict):
+            dataset_list = self.reader.read()
+            self.datasets = {}
+            for dataset in dataset_list:
+                self.datasets[dataset.title] = dataset
         order = []
-        for dset in datasets:
-            if self.dataset_type is None or dset.data_type == self.data_type:
-                if 'Log' in dset.title:
-                    position = dset.title.find('Log_') + 4
-                    order.append(int(dset.title[position:position + 3])+1)
-                else:
-                    order.append(0)
+        keys = []
+        for title, dset in self.datasets.items():
+            if isinstance(dset, sidpy.Dataset):
+                if self.dataset_type is None or dset.data_type == self.data_type:
+                    if 'Log' in title:
+                        order.append(2)
+                    else:
+                        order.append(0)
+                    keys.append(title)
         for index in np.argsort(order):
-            dset = datasets[index]
-            self.dataset_names.append('/'.join(dset.title.replace('-', '_').split('/')[-1:]))
-            self.dataset_list.append(dset)
+            self.dataset_names.append(keys[index])
+            self.dataset_list.append(keys[index] + ': ' + self.datasets[keys[index]].title)
 
     def set_dataset(self, b):
         index = self.select_image.index
-        self.dataset = self.dataset_list[index]
-        # Fnd
+        self.dataset = self.datasets[self.dataset_names[index]]
         self.dataset.title = self.dataset.title.split('/')[-1]
 
 
@@ -351,7 +361,7 @@ def open_file_dialog_qt(file_types=None):  # , multiple_files=False):
 
     # determine file types by extension
     if file_types is None:
-        file_types = 'TEM files (*.dm3 *.emi *.ndata *.h5 *.hf5);;pyNSID files (*.hf5);;QF files ( *.qf3);;' \
+        file_types = 'TEM files (*.dm3 *.emd *.ndata *.h5 *.hf5);;pyNSID files (*.hf5);;QF files ( *.qf3);;' \
                      'DM files (*.dm3 *.dm4);;Nion files (*.ndata *.h5);;All files (*)'
     elif file_types == 'pyNSID':
         file_types = 'pyNSID files (*.hf5);;TEM files (*.dm3 *.qf3 *.ndata *.h5 *.hf5);;QF files ( *.qf3);;' \
@@ -380,7 +390,6 @@ def save_dataset(dataset, filename=None,  h5_group=None):
         name of file to be opened, if filename is None, a QT file dialog will try to open
     h5_group: hd5py.Group
         not used yet
-
     """
     if filename is None:
         filename = open_file_dialog_qt()
@@ -389,37 +398,70 @@ def save_dataset(dataset, filename=None,  h5_group=None):
     path, file_name = os.path.split(filename)
     basename, _ = os.path.splitext(file_name)
 
-    if h5_group is None:
-        if 'Measurement_000' in h5_file:
-            h5_group = sidpy.hdf.prov_utils.create_indexed_group(h5_group, 'Measurement_')
-        else:
-            h5_group = h5_file.create_group('Measurement_000')
+    if isinstance(dataset, dict):
+        h5_group = save_dataset_dictionary(h5_file, dataset)
+        return h5_group
 
-        if 'Channel_000' in h5_group:
-            h5_group = sidpy.hdf.prov_utils.create_indexed_group(h5_group, 'Channel_')
-        else:
-            h5_group = h5_group.create_group('Channel_000')
+    elif isinstance(dataset, sidpy.Dataset):
+        h5_dataset = save_single_dataset(h5_file, dataset, h5_group=h5_group)
+        return h5_dataset.parent
+    else:
+        raise TypeError('Only sidpy.datasets or dictionaries can be saved with pyTEMlib')
+
+
+def save_single_dataset(h5_file, dataset, h5_group=None):
+    if h5_group is None:
+        h5_measurement_group = sidpy.hdf.prov_utils.create_indexed_group(h5_file, 'Measurement_')
+        h5_group = sidpy.hdf.prov_utils.create_indexed_group(h5_measurement_group, 'Channel_')
 
     elif isinstance(h5_group, str):
         if h5_group not in h5_file:
             h5_group = h5_file.create_group(h5_group)
         else:
-
             if h5_group[-1] == '/':
                 h5_group = h5_group[:-1]
 
             channel = h5_group.split('/')[-1]
-            h5_group = h5_group[:-len(channel)]
+            h5_measurement_group = h5_group[:-len(channel)]
             h5_group = sidpy.hdf.prov_utils.create_indexed_group(h5_group, 'Channel_')
     else:
         raise ValueError('h5_group needs to be string or None')
-    dataset.original_metadata['original_title'] = dataset.title
-    dataset.title = basename
+
     h5_dataset = pyNSID.hdf_io.write_nsid_dataset(dataset, h5_group)
     dataset.h5_dataset = h5_dataset
+    h5_dataset.file.flush()
     return h5_dataset
 
 
+def save_dataset_dictionary(h5_file, datasets):
+    h5_measurement_group = sidpy.hdf.prov_utils.create_indexed_group(h5_file, 'Measurement_')
+    for key, dataset in datasets.items():
+        if key[-1] == '/':
+            key = key[:-1]
+        if isinstance(dataset, sidpy.Dataset):
+            h5_group = h5_measurement_group.create_group(key)
+            h5_dataset = pyNSID.hdf_io.write_nsid_dataset(dataset, h5_group)
+            dataset.h5_dataset = h5_dataset
+            h5_dataset.file.flush()
+        elif isinstance(dataset, dict):
+            sidpy.hdf.hdf_utils.write_dict_to_h5_group(h5_measurement_group, dataset, key)
+        else:
+            print('could not save item ', key, 'of dataset dictionary')
+    return h5_measurement_group
+
+
+def h5_group_to_dict(group, group_dict={}):
+    if not isinstance(group, h5py.Group):
+        raise TypeError('we need a h5py group to read from')
+    if not isinstance(group_dict, dict):
+        raise TypeError('group_dict needs to be a python dictionary')
+
+    group_dict[group.name.split('/')[-1]] = dict(group.attrs)
+    for key in group.keys():
+        h5_group_to_dict(group[key], group_dict[group.name.split('/')[-1]])
+    return group_dict
+
+
 def open_file(filename=None,  h5_group=None, write_hdf_file=False):  # save_file=False,
     """Opens a file if the extension is .hf5, .ndata, .dm3 or .dm4
 
@@ -464,12 +506,25 @@ def open_file(filename=None,  h5_group=None, write_hdf_file=False):  # save_file
         datasets = reader.read()
         if len(datasets) < 1:
             print('no hdf5 dataset found in file')
-            return
+            return {}
         else:
+            dataset_dict = {}
+            for index, dataset in enumerate(datasets):
+                title = dataset.title.split('/')[2]
+                dataset.title = dataset.title.split('/')[-1]
+                dataset_dict[title] = dataset
+                if index == 0:
+                    file = datasets[0].h5_dataset.file
+                    master_group = datasets[0].h5_dataset.parent.parent.parent
+            for key in master_group.keys():
+
+                if key not in dataset_dict:
+                    dataset_dict[key] = h5_group_to_dict(master_group[key])
+                    print()
             if not write_hdf_file:
-                datasets[0].h5_dataset.file.close()
+                file.close()
                 # datasets[0].h5_dataset = None
-            return datasets[0]
+            return dataset_dict
 
         """ 
         should go to no dataset found
@@ -478,7 +533,7 @@ def open_file(filename=None,  h5_group=None, write_hdf_file=False):  # save_file
             dataset.h5_dataset = h5_group['Raw_Data']
         """
 
-    elif extension in ['.dm3', '.dm4', '.ndata', '.ndata1', '.h5', '.emi']:
+    elif extension in ['.dm3', '.dm4', '.ndata', '.ndata1', '.h5', '.emd', '.emi']:
 
         # tags = open_file(filename)
         if extension in ['.dm3', '.dm4']:
@@ -492,6 +547,8 @@ def open_file(filename=None,  h5_group=None, write_hdf_file=False):  # save_file
             except ImportError:
                 print('This file type needs hyperspy to be installed to be able to be read')
                 return
+        elif extension == '.emd':
+            reader = SciFiReaders.EMDReader(filename)
 
         else:   # extension in ['.ndata', '.h5']:
             reader = SciFiReaders.NionReader(filename)
@@ -510,32 +567,31 @@ def open_file(filename=None,  h5_group=None, write_hdf_file=False):  # save_file
                     if 'ImageData' in dset.original_metadata['ImageList']['0']:
                         if 'Data' in dset.original_metadata['ImageList']['0']['ImageData']:
                             del dset.original_metadata['ImageList']['0']['ImageData']['Data']
+            dset.original_metadata['original_title'] = dset.title
 
-        dset.filename = basename.strip().replace('-', '_')
-        # dset.original_metadata = flatten_dict(dset.original_metadata)
-        read_essential_metadata(dset)
-        if write_hdf_file:
-            filename = os.path.join(path,  dset.title+extension)
-
-            h5_filename = get_h5_filename(filename)
-            h5_file = h5py.File(h5_filename, mode='a')
-
-            if 'Measurement_000' in h5_file:
-                print('could not write dataset to file, try saving it with ft.save()')
+        if isinstance(dset, list):
+            if len(dset) < 1:
+                print('no dataset found in file')
+                return {}
             else:
-                if not isinstance(h5_group, h5py.Group):
-                    h5_group = h5_file.create_group('Measurement_000/Channel_000')
-                # dset.axes = dset._axes
-                # dset.attrs = {}
-                h5_dataset = pyNSID.hdf_io.write_nsid_dataset(dset, h5_group)
-
-                # dset.original_metadata = nest_dict(dset.original_metadata)
-
-                dset.h5_dataset = h5_dataset
-                # pyNSID.io.hdf_utils.make_nexus_compatible(h5_dataset)
+                dataset_dict = {}
+                for index, dataset in enumerate(dset):
+                    if extension == '.emd':
+                        if 'experiment' in dataset.metadata:
+                            if 'detector' in dataset.metadata['experiment']:
+                                dataset.title = dataset.metadata['experiment']['detector']
+                        dataset.filename = basename.strip()
+                        dataset.metadata['filename'] = filename
+                    dataset_dict[f'Channel_{index:03}'] = dataset
+        else:
+            dset.filename = basename.strip().replace('-', '_')
+            read_essential_metadata(dset)
+            dataset_dict = {'Channel_000': dset}
+        if write_hdf_file:
+            h5_master_group = save_dataset(dataset_dict, filename=filename)
 
         save_path(path)
-        return dset
+        return dataset_dict
     else:
         print('file type not handled yet.')
         return
@@ -552,7 +608,7 @@ def read_essential_metadata(dataset):
     """
     if not isinstance(dataset, sidpy.Dataset):
         raise TypeError("we need a sidpy.Dataset")
-
+    experiment_dictionary = {}
     if 'metadata' in dataset.original_metadata:
         if 'hardware_source' in dataset.original_metadata['metadata']:
             experiment_dictionary = read_nion_image_info(dataset.original_metadata)
@@ -560,6 +616,7 @@ def read_essential_metadata(dataset):
         experiment_dictionary = read_dm3_info(dataset.original_metadata)
     if 'experiment' not in dataset.metadata:
         dataset.metadata['experiment'] = {}
+
     dataset.metadata['experiment'].update(experiment_dictionary)
 
 
@@ -634,9 +691,6 @@ def read_nion_image_info(original_metadata):
     # print(exp_dictionary)
     if 'autostem' in exp_dictionary:
         pass
-        #print('auto')
-
-    # print(exp_dictionary.keys())
 
 
 def get_h5_filename(fname):
@@ -785,9 +839,6 @@ def add_dataset(dataset, h5_group=None):
     if hasattr(dataset, 'meta_data'):
         if 'analysis' in dataset.meta_data:
             log_group['analysis'] = dataset.meta_data['analysis']
-    if hasattr(dataset, 'structures'):
-        for structure in dataset.structures.values():
-            h5_add_crystal_structure(log_group, structure)
 
     dataset.h5_dataset = h5_dataset
     return h5_dataset
